@@ -56,15 +56,15 @@ class _TestUDP:
 
         s_transport, server = self.loop.run_until_complete(coro)
 
-        host, port, *_ = s_transport.get_extra_info('sockname')
+        remote_addr = s_transport.get_extra_info('sockname')
+        host, port, *_ = remote_addr
 
         self.assertIsInstance(server, TestMyDatagramProto)
         self.assertEqual('INITIALIZED', server.state)
         self.assertIs(server.transport, s_transport)
 
         extra = {}
-        if hasattr(socket, 'SO_REUSEPORT') and \
-                sys.version_info[:3] >= (3, 5, 1):
+        if hasattr(socket, 'SO_REUSEPORT'):
             extra['reuse_port'] = True
 
         coro = self.loop.create_datagram_endpoint(
@@ -86,6 +86,36 @@ class _TestUDP:
         # received
         self.assertEqual(8, client.nbytes)
 
+        # https://github.com/MagicStack/uvloop/issues/319
+        # uvloop should behave the same as asyncio when given remote_addr
+        transport.sendto(b'xxx', remote_addr)
+        tb.run_until(
+            self.loop, lambda: server.nbytes > 3 or client.done.done())
+        self.assertEqual(6, server.nbytes)
+        tb.run_until(self.loop, lambda: client.nbytes > 8)
+
+        # received
+        self.assertEqual(16, client.nbytes)
+
+        # reject sendto with a different port
+        with self.assertRaisesRegex(
+            ValueError, "Invalid address.*" + repr(remote_addr)
+        ):
+            bad_addr = list(remote_addr)
+            bad_addr[1] += 1
+            bad_addr = tuple(bad_addr)
+            transport.sendto(b"xxx", bad_addr)
+
+        # reject sento with unresolved hostname
+        if remote_addr[0] != lc_addr[0]:
+            with self.assertRaisesRegex(
+                ValueError, "Invalid address.*" + repr(remote_addr)
+            ):
+                bad_addr = list(remote_addr)
+                bad_addr[0] = lc_addr[0]
+                bad_addr = tuple(bad_addr)
+                transport.sendto(b"xxx", bad_addr)
+
         # extra info is available
         self.assertIsNotNone(transport.get_extra_info('sockname'))
 
@@ -100,8 +130,11 @@ class _TestUDP:
         self._test_create_datagram_endpoint_addrs(
             socket.AF_INET, ('127.0.0.1', 0))
 
-    @unittest.skipUnless(tb.has_IPv6, 'no IPv6')
-    def test_create_datagram_endpoint_addrs_ipv6(self):
+    def test_create_datagram_endpoint_addrs_ipv4_nameaddr(self):
+        self._test_create_datagram_endpoint_addrs(
+            socket.AF_INET, ('localhost', 0))
+
+    def _test_create_datagram_endpoint_addrs_ipv6(self):
         self._test_create_datagram_endpoint_addrs(
             socket.AF_INET6, ('::1', 0))
 
@@ -145,21 +178,13 @@ class _TestUDP:
             assert False, 'Can not create socket.'
 
         with sock:
-            try:
-                f = self.loop.create_datagram_endpoint(
-                    lambda: MyDatagramProto(loop=self.loop), sock=sock)
-            except TypeError as ex:
-                # asyncio in 3.5.0 doesn't have the 'sock' argument
-                if 'got an unexpected keyword argument' not in ex.args[0]:
-                    raise
-            else:
-                tr, pr = self.loop.run_until_complete(f)
-                self.assertIsInstance(pr, MyDatagramProto)
-                tr.close()
-                self.loop.run_until_complete(pr.done)
+            f = self.loop.create_datagram_endpoint(
+                lambda: MyDatagramProto(loop=self.loop), sock=sock)
+            tr, pr = self.loop.run_until_complete(f)
+            self.assertIsInstance(pr, MyDatagramProto)
+            tr.close()
+            self.loop.run_until_complete(pr.done)
 
-    @unittest.skipIf(sys.version_info < (3, 5, 1),
-                     "asyncio in 3.5.0 doesn't have the 'sock' argument")
     def test_create_datagram_endpoint_sock_unix_domain(self):
 
         class Proto(asyncio.DatagramProtocol):
@@ -250,6 +275,83 @@ class _TestUDP:
 
         self.loop.run_until_complete(run())
 
+    def test_socketpair(self):
+        peername = asyncio.Future(loop=self.loop)
+
+        class Proto(MyDatagramProto):
+            def datagram_received(self, data, addr):
+                super().datagram_received(data, addr)
+                peername.set_result(addr)
+
+        s1, s2 = socket.socketpair(socket.AF_UNIX, socket.SOCK_DGRAM, 0)
+
+        with s1, s2:
+            f = self.loop.create_datagram_endpoint(
+                lambda: Proto(loop=self.loop), sock=s1)
+            tr, pr = self.loop.run_until_complete(f)
+            self.assertIsInstance(pr, Proto)
+
+            s2.send(b'hello, socketpair')
+            addr = self.loop.run_until_complete(
+                asyncio.wait_for(peername, 1))
+            if sys.platform.startswith('linux'):
+                self.assertEqual(addr, None)
+            else:
+                self.assertEqual(addr, '')
+            self.assertEqual(pr.nbytes, 17)
+
+            if not self.is_asyncio_loop():
+                # asyncio doesn't support sendto(xx) on UDP sockets
+                # https://git.io/Jfqbw
+                data = b'from uvloop'
+                tr.sendto(data)
+                result = self.loop.run_until_complete(asyncio.wait_for(
+                    self.loop.run_in_executor(None, s2.recv, 1024),
+                    1))
+                self.assertEqual(data, result)
+
+            tr.close()
+            self.loop.run_until_complete(pr.done)
+
+    def _skip_create_datagram_endpoint_reuse_addr(self):
+        if self.implementation == 'asyncio':
+            if sys.version_info[:2] >= (3, 11):
+                raise unittest.SkipTest()
+            if (3, 8, 0) <= sys.version_info < (3, 8, 1):
+                raise unittest.SkipTest()
+            if (3, 7, 0) <= sys.version_info < (3, 7, 6):
+                raise unittest.SkipTest()
+
+    def test_create_datagram_endpoint_reuse_address_error(self):
+        # bpo-37228: Ensure that explicit passing of `reuse_address=True`
+        # raises an error, as it is not safe to use SO_REUSEADDR when using UDP
+
+        self._skip_create_datagram_endpoint_reuse_addr()
+
+        coro = self.loop.create_datagram_endpoint(
+            lambda: MyDatagramProto(loop=self.loop),
+            local_addr=('127.0.0.1', 0),
+            reuse_address=True)
+
+        with self.assertRaises(ValueError):
+            self.loop.run_until_complete(coro)
+
+    def test_create_datagram_endpoint_reuse_address_warning(self):
+        # bpo-37228: Deprecate *reuse_address* parameter
+
+        self._skip_create_datagram_endpoint_reuse_addr()
+
+        coro = self.loop.create_datagram_endpoint(
+            lambda: MyDatagramProto(loop=self.loop),
+            local_addr=('127.0.0.1', 0),
+            reuse_address=False)
+
+        with self.assertWarns(DeprecationWarning):
+            tr, pr = self.loop.run_until_complete(coro)
+
+        tr.close()
+        self.loop.run_until_complete(pr.done)
+
 
 class Test_UV_UDP(_TestUDP, tb.UVTestCase):
 
@@ -291,6 +393,16 @@ class Test_UV_UDP(_TestUDP, tb.UVTestCase):
         self.loop.run_until_complete(asyncio.sleep(0.01))
         s_transport.sendto(b'aaaa', ('127.0.0.1', 80))
 
+    @unittest.skipUnless(tb.has_IPv6, 'no IPv6')
+    def test_create_datagram_endpoint_addrs_ipv6(self):
+        self._test_create_datagram_endpoint_addrs_ipv6()
+
 
 class Test_AIO_UDP(_TestUDP, tb.AIOTestCase):
-    pass
+    @unittest.skipUnless(tb.has_IPv6, 'no IPv6')
+    @unittest.skipIf(
+        sys.version_info[:3] == (3, 7, 0),
+        'bpo-27500: bug fixed in Python 3.7.1 and above.',
+    )
+    def test_create_datagram_endpoint_addrs_ipv6(self):
+        self._test_create_datagram_endpoint_addrs_ipv6()
